ForceRequiredAddOn("Sound_Synth4");

datablock ParticleData(ExperienceBubbleParticle) {
  dragCoefficient = 4;
  gravityCoefficient = -1;
  inheritedVelFactor = 0;
  constantAcceleration = 0;
  lifetimeMS = 500;
  lifetimeVarianceMS = 250;
  textureName = "base/data/particles/bubble";
  spinSpeed = 0;
  spinRandomMin = 0;
  spinRandomMax = 0;
  colors[0] = "1 1 0 0.3";
  colors[1] = "0.3 0.9 0.9 0.6";
  colors[2] = "0.3 0.4 1 0";
  sizes[0] = 0.1;
  sizes[1] = 0.6;
  sizes[2] = 0.2;
  times[0] = 0;
  times[1] = 0.1;
  times[2] = 1;
  useInvAlpha = false;
};

datablock ParticleEmitterData(ExperienceBubbleEmitter) {
  uiName = "";
  ejectionPeriodMS = 25;
  periodVarianceMS = 20;
  ejectionVelocity = 0;
  velocityVariance = 0;
  ejectionOffset = 0.1;
  thetaMin = 0;
  thetaMax = 180;
  phiReferenceVel = 0;
  phiVariance = 360;
  overrideAdvance = false;
  particles = "ExperienceBubbleParticle";
};

datablock ShapeBaseImageData(ExperienceBubbleImage) {
  shapeFile = "base/data/shapes/empty.dts";
  emap = false;

  stateName[0] = "Fire";
  stateTransitionOnTimeout[0] = "Fire";
  stateWaitForTimeout[0] = true;
  stateTimeoutValue[0] = 1000;
  stateEmitter[0] = ExperienceBubbleEmitter;
  stateEmitterTime[0] = 1000;
};

if(isPackage(ExperienceBubbles))
   deActivatePackage(ExperienceBubbles);
   
package ExperienceBubbles {
   function ItemData::onAdd(%this, %obj) {
     parent::onAdd(%this, %obj);
     
     if(%this.uiname $="" && !isObject(%obj.getMountedImage(0)) && %this == nameToId(ExperienceBubbleItem))
         %obj.mountImage(ExperienceBubbleImage, 0);
   }
   
   function GameConnection::onDeath(%this, %killerPlayer, %killerClient, %damageType, %damageArea) {
      if(%this.player && %this.score > 0) {
         %pos = %this.player.getHackPosition();
      } else {
         return Parent::onDeath(%this, %killerPlayer, %killerClient, %damageType, %damageArea);
      }
      %score = %this.score;
      
      %log = mCeil(%this.score / 10);
      
      for(%i = 0; %i < %score; %i += %log) {
         %amount = mClamp(%log+getRandom(0, 5), 0, %this.score);
         if(%amount == 0)
            continue;
         %theta = getRandom()*3.14159*2;
         CreateExperienceBubble(%amount, %pos, vectorScale(mCos(%theta) SPC mSin(%theta) SPC mCos(getRandom()*3.1415),10), 10000);
         %this.score -= %amount;
      }
      return Parent::onDeath(%this, %killerPlayer, %killerClient, %damageType, %damageArea);
   }
};
activatepackage(ExperienceBubbles);

datablock ItemData(ExperienceBubbleItem) {
   uiname = "";
   shapefile = "base/data/shapes/empty.dts";
   canPickup = 0;
   canDrop = 1;
   gravityMod = 0.6;
   mass = 1000;
   friction = 0.6;
   
};

function ExperienceBubbleItem::onPickup(%this, %obj, %player, %slot) {
   %player.client.score += %obj.score;
   %sound = getRandom(0, 11);
   if(strLen(%sound) == 1)
      %sound = 0 @ %sound;
   serverPlay3d(Synth_ @ %sound @ _Sound, %obj.getPosition());
   %obj.schedule(0, delete);
   return;
}

function CreateExperienceBubble(%score, %pos, %vel, %life) {
   
   %item = new Item() {
      datablock = ExperienceBubbleItem;
      position = %pos;
      velocity = %vel;
      canPickup = 0;
      canDrop = 1;
      score = %score;
      static = 0;
      scale = "1 1 1";
   };
   
   %item.setVelocity(%vel);
   %item.setShapeNameDistance(10);
   %item.setShapeName(%score);
   %item.schedule(%life, delete);
   return %item;
}

function Player::mPopDamage(%obj,%pos,%damage)
{
   %ang = mDegToRad(getRandom(0,359));
   %vel = vectorScale(mCos(%ang) SPC mSin(%ang) SPC 0.5+getRandom(),2);
   mCreateDamageItem(%damage,%obj.getHackPosition(),%vel,2000,200);
}

registerOutputEvent(fxDTSBrick, "SpawnExperience", "int 1 300000 1\tvector\tint 5 30 30");
function fxDTSBrick::SpawnExperience(%obj, %size, %vel, %life) {
   CreateExperienceBubble(%size, %obj.getPosition(), %vel, %life*1000);
}